/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2013, Red Hat, Inc., and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.wildfly.clustering.marshalling.jboss;

import java.io.IOException;

import org.jboss.marshalling.Marshaller;
import org.jboss.marshalling.Unmarshaller;

/**
 * A marshalling context for use with a {@link MarshalledValue}.
 * @author Paul Ferraro
 */
public interface MarshallingContext {
    ClassLoader getClassLoader();

    int getCurrentVersion();

    Unmarshaller createUnmarshaller(int version) throws IOException;

    Marshaller createMarshaller(int version) throws IOException;

    /**
     * Indicates whether the specified object can be marshalled.
     * @param object an object to be marshalled
     * @return true, if the specified object can be marshalled, false otherwise
     */
    boolean isMarshallable(Object object);
}
